/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.tools.jmx.api;

import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.MBean.PETALSADMIN_OBJECT_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.PETALS14_RETRIEVE_CONTAINERS_CONFIGURATION;
import static org.ow2.petals.tools.jmx.api.IPetalsAdminService.OperationNames.RETRIEVE_CONTAINERS_CONFIGURATION;

import java.io.IOException;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.petals.tools.jmx.api.exception.AdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.AdminServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.ComponentComponentDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.ComponentErrorException;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.EndpointServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.EndpointServiceServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.InstallationServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.MonitoringDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.MonitoringServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.PerformActionDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.TransportMonitoringDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.TransportMonitoringServiceErrorException;

/**
 * The Petals JMX Client API
 *
 * @author cdeneux - Capgemini Sud
 * @author cgirodengo - Capgemini Sud
 */
public final class PetalsJMXClient {

    private final static String PETALS_DOMAIN = "Petals";

    private final static String JAVA_LANG_DOMAIN = "java.lang";

	private PetalsJMXConnection jmxConnection = null;

	private boolean isPetalsServer20Older = false;

	/**
	 * Creates a new instance of {@link PetalsJMXClient}
	 *
	 * @param host
	 * @param jndiPort
	 * @param username
	 * @param password
	 * @throws ConnectionErrorException
	 */
	public PetalsJMXClient(String host, Integer jndiPort, String username,
			String password) throws ConnectionErrorException {
		this.jmxConnection = new PetalsJMXConnection(host, jndiPort, username,
				password);

		// Check PEtALS server version
		try {
            InternalCheckPetalsVersionServiceClient isc = new InternalCheckPetalsVersionServiceClient(PETALS_DOMAIN, jmxConnection
                    .getMBeanServerConnection());
            this.isPetalsServer20Older = isc.isPetalsServer20Older();
        } catch (PetalsAdminDoesNotExistException e) {
            // TODO Auto-generated catch block
        } catch (PetalsAdminServiceErrorException e) {
            // TODO Auto-generated catch block
        } catch (PerformActionErrorException e) {
            // TODO Auto-generated catch block
        }
	}

	/**
	 * Retrieve the Petals Admin service client API
	 *
	 * @return the Petals Admin service client API
	 * @throws PetalsAdminDoesNotExistException
	 * @throws PetalsAdminServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public PetalsAdminServiceClient getPetalsAdminServiceClient()
			throws PetalsAdminDoesNotExistException,
			PetalsAdminServiceErrorException, ConnectionErrorException {

		return new PetalsAdminServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection(), this.isPetalsServer20Older);

	}

	/**
	 * Retrieve the Admin service client API
	 *
	 * @return the Admin service client API
	 * @throws AdminDoesNotExistException
	 * @throws AdminServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public AdminServiceClient getAdminServiceClient()
			throws AdminDoesNotExistException, AdminServiceErrorException,
			ConnectionErrorException {

		return new AdminServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}

	/**
	 * Retrieve the installation service client API
	 *
	 * @return the installation service client API
	 * @throws InstallationServiceDoesNotExistException
	 * @throws InstallationServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public PetalsInstallationServiceClient getInstallationServiceClient()
			throws InstallationServiceDoesNotExistException,
			InstallationServiceErrorException, ConnectionErrorException {

		return new PetalsInstallationServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}

	/**
	 * Retrieve the deployment service client API
	 *
	 * @return the deployment service client API
	 * @throws DeploymentServiceDoesNotExistException
	 * @throws DeploymentServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public PetalsDeploymentServiceClient getDeploymentServiceClient()
			throws DeploymentServiceDoesNotExistException,
			DeploymentServiceErrorException, ConnectionErrorException {

		return new PetalsDeploymentServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}

	/**
	 * Retrieve the component client API associated to a JBI component.
	 *
	 * @param componentName
	 *            Component name
	 * @return the component client API
	 * @throws ComponentDoesNotExistException
	 * @throws ComponentErrorException
	 * @throws ConnectionErrorException
	 */
	public PetalsComponentComponentClient getComponentComponentClient(
			String componentName)
			throws ComponentComponentDoesNotExistException,
			ComponentErrorException, ConnectionErrorException {

		return new PetalsComponentComponentClient(PETALS_DOMAIN, componentName,
				jmxConnection.getMBeanServerConnection());

	}

	/**
	 * Retrieve the component installer client API associated to a JBI
	 * component.
	 *
	 * @param componentName
	 *            Component name
	 * @return the installer component client API
	 * @throws InstallationServiceDoesNotExistException
	 * @throws InstallationServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public PetalsInstallerComponentClient getInstallerComponentClient(
			String componentName)
			throws InstallerComponentDoesNotExistException,
			ComponentErrorException, ConnectionErrorException {

		return new PetalsInstallerComponentClient(PETALS_DOMAIN, componentName,
				jmxConnection.getMBeanServerConnection());

	}

	/**
	 * Retrieve the EndpointRegistry client API
	 *
	 * @return the EndpointRegistry component client API
	 * @throws ConnectionErrorException
	 * @throws EndpointServiceServiceErrorException
	 * @throws EndpointServiceDoesNotExistException
	 */
	public EndpointRegistryClient getEndpointRegistryClient()
			throws ConnectionErrorException,
			EndpointServiceDoesNotExistException,
			EndpointServiceServiceErrorException {

		return new EndpointRegistryClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection(), this.isPetalsServer20Older);

	}

	/**
	 * Retrieve the monitoring service client API
	 *
	 * @return the MonitoringService component client API
	 * @throws MonitoringDoesNotExistException
	 * @throws MonitoringServiceErrorException
	 * @throws ConnectionErrorException
	 */
	public MonitoringServiceClient getMonitoringServiceClient()
			throws ConnectionErrorException,
			MonitoringDoesNotExistException,
			MonitoringServiceErrorException {

		return new MonitoringServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}

	/**
	 * Retrieve the system monitoring service client API
	 *
	 * @return the SystemMonitoring component client API
	 * @throws ConnectionErrorException
	 * @throws SystemMonitoringServiceErrorException
	 * @throws SystemMonitoringDoesNotExistException
	 */
	public SystemMonitoringServiceClient getSystemMonitoringServiceClient()
			throws ConnectionErrorException,
			SystemMonitoringDoesNotExistException,
			SystemMonitoringServiceErrorException {

		return new SystemMonitoringServiceClient(JAVA_LANG_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}

	/**
	 * Retrieve the transport monitoring service client API
	 *
	 * @return the TransportMonitoring component client API
	 * @throws InstallationServiceDoesNotExistException
	 * @throws ConnectionErrorException
	 * @throws TransportMonitoringServiceErrorException
	 * @throws TransportMonitoringDoesNotExistException
	 */
	public TransportMonitoringServiceClient getTransportMonitoringServiceClient()
			throws ConnectionErrorException,
			TransportMonitoringDoesNotExistException,
			TransportMonitoringServiceErrorException {

		return new TransportMonitoringServiceClient(PETALS_DOMAIN, jmxConnection
				.getMBeanServerConnection());

	}
    
    /**
     * Release the connection to the JMX server
     * 
     * @throws ConnectionErrorException
     */
    public void disconnect() throws ConnectionErrorException {
        if (this.jmxConnection != null) {
            this.jmxConnection.disconnect();
        }
    }

	private class InternalCheckPetalsVersionServiceClient extends PetalsAbstractServiceClient {
	    /**
	     * Creates a new instance of {@link InternalCheckPetalsVersionServiceClient}
	     *
	     * @param petalsDomain
	     * @param beanServerConnection
	     * @throws PetalsAdminDoesNotExistException
	     * @throws PetalsAdminServiceErrorException
	     */
	    public InternalCheckPetalsVersionServiceClient(String petalsDomain, MBeanServerConnection beanServerConnection)
	            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException {
	        super(petalsDomain, beanServerConnection);

	        try {
	            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(
	                    PETALSADMIN_OBJECT_NAME, null);
	            if (objNames != null && objNames.size() == 1) {
	                this.mbeanName = objNames.iterator().next();
	            } else {
	                throw new PetalsAdminDoesNotExistException();
	            }
	        } catch (IOException e) {
	            throw new PetalsAdminServiceErrorException(e);
	        }
	    }

	    /**
	     * Check the Petals server version.
	     *
	     * @throws PerformActionErrorException
	     */
	    public boolean isPetalsServer20Older()
	            throws PerformActionErrorException {
	        boolean isPetalsServer20Older = false;

            try {
	           this.performAction(RETRIEVE_CONTAINERS_CONFIGURATION, new Object[0], new String[0]);
	        } catch (PerformActionDoesNotExistException e) {
                // Perhaps the petals server version is 1.4.x
                this.performAction(PETALS14_RETRIEVE_CONTAINERS_CONFIGURATION, new Object[0], new String[0]);
                isPetalsServer20Older = true;
	        }

            return isPetalsServer20Older;
	    }
	}
}
